@@ -20,6 +20,7 @@ import time
 from urllib import parse as urlparse
 
 import requests
+from urllib3.exceptions import InsecureRequestWarning
 
 from sushy import exceptions
 from sushy.taskmonitor import TaskMonitor
@@ -28,21 +29,19 @@ from sushy import utils
 LOG = logging.getLogger(__name__)
 
 
-_SERVER_SIDE_RETRIES = 10
-_SERVER_SIDE_RETRY_DELAY = 3
-
-
 class Connector(object):
 
     def __init__(
             self, url, username=None, password=None, verify=True,
-            response_callback=None):
+            response_callback=None, server_side_retries=0,
+            server_side_retries_delay=0):
         self._url = url
         self._verify = verify
         self._session = requests.Session()
-        self._session.verify = self._verify
         self._response_callback = response_callback
         self._auth = None
+        self._server_side_retries = server_side_retries
+        self._server_side_retries_delay = server_side_retries_delay
 
         # NOTE(TheJulia): In order to help prevent recursive post operations
         # by allowing us to understand that we should stop authentication.
@@ -52,15 +51,6 @@ class Connector(object):
         # By default, we ask HTTP server to shut down HTTP connection we've
         # just used.
         self._session.headers['Connection'] = 'close'
-        # NOTE(TheJulia): Depending on the BMC, offering compression as an
-        # acceptable response changes the ETag behavior to offering an
-        # automatic "weak" ETag response, which is appropriate because the
-        # body content *may* not be a byte for byte match given compression.
-        # Overall, the value of compression is less than the value of concise
-        # interaction with the BMC. Setting to identity basically means
-        # "without modification or compression". By default, python-requests
-        # indicates responses can be compressed.
-        self._session.headers['Accept-Encoding'] = 'identity'
 
         if username or password:
             LOG.warning('Passing username and password to Connector is '
@@ -69,6 +59,13 @@ class Connector(object):
                         'be removed in the future')
             self.set_http_basic_auth(username, password)
 
+        if not self._verify:
+            # As the user specifically needs to opt out of certificate
+            # validation the user is aware of the security implications
+            # and does not need to be overwhelmed by the urllib3 warnings
+            requests.packages.urllib3.disable_warnings(
+                category=InsecureRequestWarning)
+
     def set_auth(self, auth):
         """Sets the authentication mechanism for our connector."""
         self._auth = auth
@@ -86,8 +83,22 @@ class Connector(object):
         """Close this connector and the associated HTTP session."""
         self._session.close()
 
+    def check_retry_on_exception(self, exception_msg):
+        """Checks whether retry on exception is required."""
+        retry = False
+        exc_str = str(exception_msg)
+        if 'SYS518' in exc_str:
+            LOG.debug('iDRAC is not yet ready after previous operation. '
+                      'Error: %(err)s', {'err': exc_str})
+            retry = True
+        elif 'iLO' in exc_str and 'InvalidOperationForSystemState' in exc_str:
+            LOG.debug('iLO is not ready after previous operation. '
+                      'Error: %(error)s', {'err': exc_str})
+            retry = True
+        return retry
+
     def _op(self, method, path='', data=None, headers=None, blocking=False,
-            timeout=60, server_side_retries=_SERVER_SIDE_RETRIES,
+            timeout=60, server_side_retries_left=None, allow_reauth=True,
             **extra_session_req_kwargs):
         """Generic RESTful request handler.
 
@@ -95,9 +106,18 @@ class Connector(object):
             PUT, PATCH, etc...
         :param path: The sub-URI or absolute URL path to the resource.
         :param data: Optional JSON data.
-        :param headers: Optional dictionary of headers.
+        :param headers: Optional dictionary of headers. Use None value
+                        to remove a default header.
         :param blocking: Whether to block for asynchronous operations.
-        :param timeout: Max time in seconds to wait for blocking async call.
+        :param timeout: Max time in seconds to wait for blocking async call or
+                        for requests library to connect and read. If a custom
+                        timeout for requests is provided in
+                        extra_session_req_kwargs, it will be used instead for
+                        those calls.
+        :param server_side_retries_left: Remaining retries. If not provided
+            will use limit provided by instance's server_side_retries
+        :param allow_reauth: Whether to allow refreshing the authentication
+            token.
         :param extra_session_req_kwargs: Optional keyword argument to pass
          requests library arguments which would pass on to requests session
          object.
@@ -105,14 +125,30 @@ class Connector(object):
         :raises: ConnectionError
         :raises: HTTPError
         """
+        if server_side_retries_left is None:
+            server_side_retries_left = self._server_side_retries
+
         url = path if urlparse.urlparse(path).netloc else urlparse.urljoin(
             self._url, path)
-        headers = headers or {}
+        headers = (headers or {}).copy()
         lc_headers = [k.lower() for k in headers]
         if data is not None and 'content-type' not in lc_headers:
             headers['Content-Type'] = 'application/json'
         if 'odata-version' not in lc_headers:
             headers['OData-Version'] = '4.0'
+        # NOTE(TheJulia): Depending on the BMC, offering compression as an
+        # acceptable response changes the ETag behavior to offering an
+        # automatic "weak" ETag response, which is appropriate because the
+        # body content *may* not be a byte for byte match given compression.
+        # Overall, the value of compression is less than the value of concise
+        # interaction with the BMC. Setting to identity basically means
+        # "without modification or compression". By default, python-requests
+        # indicates responses can be compressed.
+        if 'accept-encoding' not in lc_headers:
+            headers['Accept-Encoding'] = 'identity'
+        # Allow removing default headers
+        headers = {k: v for k, v in headers.items() if v is not None}
+
         # TODO(lucasagomes): We should mask the data to remove sensitive
         # information
         LOG.debug('HTTP request: %(method)s %(url)s; headers: %(headers)s; '
@@ -126,6 +162,8 @@ class Connector(object):
         try:
             response = self._session.request(method, url, json=data,
                                              headers=headers,
+                                             verify=self._verify,
+                                             timeout=timeout,
                                              **extra_session_req_kwargs)
         except requests.exceptions.RequestException as e:
             # Capture any general exception by looking for the parent
@@ -153,6 +191,10 @@ class Connector(object):
                 LOG.error('Authentication to the session service failed. '
                           'Please check credentials and try again.')
                 raise
+            if not allow_reauth:
+                LOG.error("Failure occurred while attempting to retry "
+                          "request after refreshing the session: %s", e)
+                raise
             if self._auth is not None:
                 # self._session.auth value is only set when basic auth is used
                 if self._session.auth is not None:
@@ -174,21 +216,17 @@ class Connector(object):
                                     're-authenticate.')
                         self._auth.authenticate()
                 except exceptions.AccessError as refresh_exc:
-                    LOG.error("A failure occured while attempting to refresh "
+                    LOG.error("A failure occurred while attempting to refresh "
                               "the session. Error: %s", refresh_exc.message)
                     raise
                 LOG.debug("Authentication refreshed successfully, "
                           "retrying the call.")
-                try:
-                    response = self._session.request(
-                        method, url, json=data,
-                        headers=headers,
-                        **extra_session_req_kwargs)
-                except exceptions.HTTPError as retry_exc:
-                    LOG.error("Failure occured while attempting to retry "
-                              "request after refreshing the session. Error: "
-                              "%s", retry_exc.message)
-                    raise
+                return self._op(
+                    method, path, data=data, headers=headers,
+                    blocking=blocking, timeout=timeout,
+                    server_side_retries_left=server_side_retries_left,
+                    allow_reauth=False,
+                    **extra_session_req_kwargs)
             else:
                 if method == 'GET' and url.endswith('SessionService'):
                     LOG.debug('HTTP GET of SessionService failed %s, '
@@ -199,18 +237,57 @@ class Connector(object):
                               "%s", e.message)
                 raise
         except exceptions.ServerSideError as e:
-            if method.lower() != 'get' or server_side_retries <= 0:
-                raise
-            else:
+            if ((method.lower() == 'get'
+                or self.check_retry_on_exception(e.message))
+                    and server_side_retries_left > 0):
                 LOG.warning('Got server side error %s in response to a '
-                            'GET request, retrying after %d seconds',
-                            e, _SERVER_SIDE_RETRY_DELAY)
-                time.sleep(_SERVER_SIDE_RETRY_DELAY)
-                return self._op(method, path, data=data, headers=headers,
-                                blocking=blocking, timeout=timeout,
-                                server_side_retries=server_side_retries - 1,
-                                **extra_session_req_kwargs)
-
+                            'request, retrying after %d seconds. Retries '
+                            'left %d.',
+                            e, self._server_side_retries_delay,
+                            server_side_retries_left)
+                time.sleep(self._server_side_retries_delay)
+                server_side_retries_left -= 1
+                return self._op(
+                    method, path, data=data, headers=headers,
+                    blocking=blocking, timeout=timeout,
+                    server_side_retries_left=server_side_retries_left,
+                    **extra_session_req_kwargs)
+            else:
+                raise
+        except exceptions.BadRequestError as e:
+            if (method.lower() != 'get'
+                    and self.check_retry_on_exception(e.message)
+                    and server_side_retries_left > 0):
+                LOG.warning('Server has indicated a BadRequest for %s but '
+                            'the response payload is a known retriable '
+                            'condition and we will retry in %d seconds. '
+                            'Retries left  %d.',
+                            e, self._server_side_retries_delay,
+                            server_side_retries_left)
+                time.sleep(self._server_side_retries_delay)
+                server_side_retries_left -= 1
+                return self._op(
+                    method, path, data=data, headers=headers,
+                    blocking=blocking, timeout=timeout,
+                    server_side_retries_left=server_side_retries_left,
+                    **extra_session_req_kwargs)
+            else:
+                raise
+        except exceptions.NotAcceptableError as e:
+            # NOTE(dtantsur): some HPE Gen 10 Plus machines do not allow
+            # identity encoding when fetching registries.
+            if (method.lower() == 'get'
+                    and headers.get('Accept-Encoding') == 'identity'):
+                LOG.warning('Server has indicated a NotAcceptable for %s, '
+                            'retrying without identity encoding', e)
+                headers = dict(headers, **{'Accept-Encoding': None})
+                return self._op(
+                    method, path, data=data, headers=headers,
+                    blocking=blocking, timeout=timeout,
+                    server_side_retries_left=server_side_retries_left,
+                    **extra_session_req_kwargs)
+            else:
+                raise
         if blocking and response.status_code == 202:
             if not response.headers.get('Location'):
                 m = ('HTTP response for %(method)s request to %(url)s '
