@@ -25,7 +25,9 @@ class SushyError(Exception):
 
     message = None
 
-    def __init__(self, **kwargs):
+    def __init__(self, message=None, **kwargs):
+        if message is not None:
+            self.message = message
         if self.message and kwargs:
             self.message = self.message % kwargs
 
@@ -91,7 +93,11 @@ class HTTPError(SushyError):
     detail = None
     """Error message defined in the Redfish specification, if present."""
 
-    message = ('HTTP %(method)s %(url)s returned code %(code)s. %(error)s')
+    message = ('HTTP %(method)s %(url)s returned code %(code)s. %(error)s '
+               'Extended information: %(ext_info)s')
+
+    extended_info = None
+    """Extended information provided in the response."""
 
     def __init__(self, method, url, response):
         self.status_code = response.status_code
@@ -107,26 +113,34 @@ class HTTPError(SushyError):
             self.body = body.get('error', {})
             self.code = self.body.get('code', 'Base.1.0.GeneralError')
             self.detail = self.body.get('message')
-            ext_info = self.body.get('@Message.ExtendedInfo', [{}])
-            index = self._get_most_severe_msg_index(ext_info)
-            self.detail = ext_info[index].get('Message', self.detail)
-            error = '%s: %s extended: %s' % (
-                self.code, self.detail or 'unknown error', ext_info or None)
-
+            self.extended_info = self.body.get('@Message.ExtendedInfo')
+            message = self._get_most_severe_msg(self.extended_info or [{}])
+            self.detail = message or self.detail
+            error = '%s: %s' % (self.code, self.detail or 'unknown error.')
         kwargs = {'method': method, 'url': url, 'code': self.status_code,
-                  'error': error}
+                  'error': error, 'ext_info': self.extended_info}
         LOG.debug('HTTP response for %(method)s %(url)s: '
-                  'status code: %(code)s, error: %(error)s', kwargs)
+                  'status code: %(code)s, error: %(error)s, '
+                  'extended: %(ext_info)s', kwargs)
         super(HTTPError, self).__init__(**kwargs)
 
     @staticmethod
-    def _get_most_severe_msg_index(extended_info):
+    def _get_most_severe_msg(extended_info):
+        if not isinstance(extended_info, list):
+            return extended_info.get('Message', None)
         if len(extended_info) > 0:
             for sev in ['Critical', 'Warning']:
                 for i, m in enumerate(extended_info):
                     if m.get('Severity') == sev:
-                        return i
-        return 0
+                        return m.get('Message')
+
+    @property
+    def related_properties(self):
+        """List of properties related to the error."""
+        try:
+            return self.extended_info[0]['RelatedProperties']
+        except (IndexError, KeyError, TypeError):
+            return []
 
 
 class BadRequestError(HTTPError):
@@ -146,6 +160,10 @@ class AccessError(HTTPError):
     pass
 
 
+class NotAcceptableError(HTTPError):
+    pass
+
+
 class MissingXAuthToken(HTTPError):
     message = ('No X-Auth-Token returned from remote host when '
                'attempting to establish a session. Error: %(error)s')
@@ -162,6 +180,8 @@ def raise_for_response(method, url, response):
     elif response.status_code in (http_client.UNAUTHORIZED,
                                   http_client.FORBIDDEN):
         raise AccessError(method, url, response)
+    elif response.status_code == http_client.NOT_ACCEPTABLE:
+        raise NotAcceptableError(method, url, response)
     elif response.status_code >= http_client.INTERNAL_SERVER_ERROR:
         raise ServerSideError(method, url, response)
     else:
