@@ -423,9 +423,13 @@ class JsonDataReader(AbstractDataReader):
     def get_data(self):
         """Gets JSON file from URI directly"""
         data = self._conn.get(path=self._path)
-
-        json_data = data.json() if data.content else {}
-
+        try:
+            json_data = data.json() if data.content else {}
+        except Exception as exc:
+            LOG.error("Unable to parse JSON in response. %(exc)s. The server "
+                      "returned:\n%(data)s",
+                      {'exc': exc, 'data': data.content})
+            raise
         return FieldData(data.status_code, data.headers, json_data)
 
 
@@ -593,8 +597,13 @@ class ResourceBase(object, metaclass=abc.ABCMeta):
 
         for key, registry in registries.items():
             if (registry
-                    and self._attribute_registry in (key, registry.identity)):
-                if language != registry.language:
+                    and identity in (key, registry.identity)):
+                # NOTE(iurygregory): some registries may have "en-US"
+                # as their language, in this case we can check if the
+                # registry language starts with the requested language.
+                registry_language = registry.language.lower().split('-', 1)[0]
+                if (language != registry.language
+                        and language.lower() != registry_language):
                     LOG.debug('Found %(descr)s but its language %(reg_lang)s '
                               'does not match the requested %(lang)s',
                               {'descr': description,
