@@ -19,12 +19,15 @@
 import collections
 import logging
 
+from dateutil import parser
+
 from sushy import exceptions
 from sushy.resources import base
 from sushy.resources.chassis import chassis
 from sushy.resources import common
 from sushy.resources import constants as res_cons
 from sushy.resources.manager import manager
+from sushy.resources.manager import virtual_media
 from sushy.resources import settings
 from sushy.resources.system import bios
 from sushy.resources.system import constants as sys_cons
@@ -56,6 +59,25 @@ class BootField(base.CompositeField):
 
     target = base.MappedField('BootSourceOverrideTarget', sys_cons.BootSource)
 
+    http_boot_uri = base.Field('HttpBootUri')
+
+
+class BootProgressField(base.CompositeField):
+
+    last_boot_seconds_count = base.Field('LastBootTimeSeconds',
+                                         adapter=utils.int_or_none)
+    """The number of seconds the last boot took to reach OSRunning."""
+
+    last_state = base.MappedField('LastState', sys_cons.BootProgressStates)
+    """The last recorded boot progress states."""
+
+    last_state_updated_at = base.Field('LastStateTime',
+                                       adapter=parser.parse)
+    """The date-time value when the last state field was updated."""
+
+    oem_last_state = base.Field('OemLastState')
+    """The OEM last state time to describe OEM specific state information."""
+
 
 class MemorySummaryField(base.CompositeField):
     health = base.Field(['Status', 'HealthRollup'])
@@ -138,6 +160,9 @@ class System(base.ResourceBase):
 
     _actions = ActionsField('Actions', required=True)
 
+    boot_progress = BootProgressField('BootProgress')
+    """The last updated boot progress indicator"""
+
     def __init__(self, connector, identity, redfish_version=None,
                  registries=None, root=None):
         """A class representing a ComputerSystem
@@ -212,7 +237,8 @@ class System(base.ResourceBase):
         return {v for v in sys_cons.BootSource
                 if v.value in self.boot.allowed_values}
 
-    def set_system_boot_options(self, target=None, enabled=None, mode=None):
+    def set_system_boot_options(self, target=None, enabled=None, mode=None,
+                                http_boot_uri=None):
         """Set boot source and/or boot frequency and/or boot mode.
 
         Set the boot source and/or boot frequency and/or boot mode to use
@@ -220,10 +246,19 @@ class System(base.ResourceBase):
 
         :param target: The target boot source,
             a :py:class:`sushy.BootSource` value. Optional.
-        :param enabled: How long the override be enabled,
+        :param enabled: How long the override is enabled,
             a :py:class:`sushy.BootSourceOverrideEnabled` value. Optional.
         :param mode: The boot mode,
             a :py:class:`sushy.BootSourceOverrideMode` value. Optional.
+        :param http_boot_uri: The requested HTTP Boot URI to transmit to the
+            BMC. Only valid when BootSourceOverrideTarget is set to UefiHTTP,
+            when utilizing the ``target`` parameter. If no value is supplied,
+            and the target is set to UefiHTTP, then an empty value will be
+            sent to the BMC to remove any prior setting, allowing the host
+            to load configuration from DHCP.
+            If not explicitly set, any value will be removed from a BMC when
+            UefiHttp boot is not engaged.
+
         :raises: InvalidParameterValueError, if any information passed is
             invalid.
         """
@@ -235,6 +270,7 @@ class System(base.ResourceBase):
             settings_boot_section = settings_resp.json().get('Boot', {})
         else:
             settings_resp = None
+            settings_boot_section = {}
 
         if target is not None:
             valid_targets = self.get_allowed_system_boot_source_values()
@@ -293,6 +329,25 @@ class System(base.ResourceBase):
             else:
                 data['Boot']['BootSourceOverrideMode'] = fishy_mode
 
+        if target == sys_cons.BootSource.UEFI_HTTP:
+            # The http_boot_uri value *can* be set independently of the
+            # target, but the BMC will just ignore it unless the target
+            # is set. So we should only, and explicitly set it when we've
+            # been requested to boot from UefiHTTP.
+            if not http_boot_uri:
+                # This should clear out any old entries, as no URI translates
+                # to the intent of "use whatever the dhcp server says".
+                data['Boot']['HttpBootUri'] = None
+            else:
+                # Explicitly set the URI.
+                data['Boot']['HttpBootUri'] = http_boot_uri
+        elif not http_boot_uri:
+            # We're not doing boot from URL, we should cleanup any setting
+            # which may be from a prior step/call.
+            if settings_boot_section.get('HttpBootUri'):
+                # If the setting is present, and has any value, unset it.
+                data['Boot']['HttpBootUri'] = None
+
         # TODO(lucasagomes): Check the return code and response body ?
         #                    Probably we should call refresh() as well.
         if settings_data.get('Boot'):
@@ -342,9 +397,10 @@ class System(base.ResourceBase):
                 parameter='state', value=state,
                 valid_values=' ,'.join(i.value for i in res_cons.IndicatorLED))
 
+        etag = self._get_etag()
         data = {'IndicatorLED': state}
 
-        self._conn.patch(self.path, data=data)
+        self._conn.patch(self.path, data=data, etag=etag)
         self.invalidate()
 
     def _get_processor_collection_path(self):
@@ -451,9 +507,21 @@ class System(base.ResourceBase):
         :raises: MissingAttributeError if '@odata.id' field is missing.
         :returns: A list of `Manager` instances
         """
-        paths = utils.get_sub_resource_path_by(
-            self, ["Links", "ManagedBy"], is_collection=True)
-
+        try:
+            paths = utils.get_sub_resource_path_by(
+                self, ["Links", "ManagedBy"], is_collection=True)
+        except exceptions.MissingAttributeError as exc_orig:
+            LOG.warning('Unable to find ManagedBy attribute for System %s, '
+                        'retrying with Managers attribute', self.identity)
+            try:
+                paths = utils.get_sub_resource_path_by(
+                    self, ["Links", "Managers"], is_collection=True)
+            except exceptions.MissingAttributeError:
+                LOG.error('Both ManagedBy and Managers attributes missing for '
+                          'System %s, aborting', self.identity)
+                # NOTE(janders) last_error may record only Managers and not
+                # ManagedBy MissingAttributeError with this approach
+                raise exc_orig
         return [manager.Manager(self._conn, path,
                                 redfish_version=self.redfish_version,
                                 registries=self.registries, root=self.root)
@@ -493,6 +561,18 @@ class System(base.ResourceBase):
             redfish_version=self.redfish_version,
             registries=self.registries, root=self.root)
 
+    @property
+    @utils.cache_it
+    def virtual_media(self):
+        """Property to reference `VirtualMedia` instance
+
+        :returns: A `VirtualMediaCollection` instance.
+        """
+        return virtual_media.VirtualMediaCollection(
+            self._conn, utils.get_sub_resource_path_by(self, 'VirtualMedia'),
+            redfish_version=self.redfish_version, registries=self.registries,
+            root=self.root)
+
 
 class SystemCollection(base.ResourceCollectionBase):
 
