@@ -20,9 +20,9 @@ from http import client as http_client
 import logging
 
 from sushy.resources import base
-from sushy.resources import mappings as res_maps
+from sushy.resources import constants as res_cons
 from sushy.resources.registry import message_registry
-from sushy.resources.taskservice import mappings as task_maps
+from sushy.resources.taskservice import constants as ts_cons
 from sushy import utils
 
 
@@ -53,17 +53,17 @@ class Task(base.ResourceBase):
     percent_complete = base.Field('PercentComplete', adapter=utils.int_or_none)
     """Percentage complete of the Task"""
 
-    task_state = base.MappedField('TaskState', task_maps.TASK_STATE_VALUE_MAP)
+    task_state = base.MappedField('TaskState', ts_cons.TaskState)
     """The Task state"""
 
-    task_status = base.MappedField('TaskStatus', res_maps.HEALTH_VALUE_MAP)
+    task_status = base.MappedField('TaskStatus', res_cons.Health)
     """The Task status"""
 
     messages = base.MessageListField("Messages")
     """List of :class:`.MessageListField` with messages from the Task"""
 
     def __init__(self, connector, identity, redfish_version=None,
-                 registries=None, json_doc=None):
+                 registries=None, json_doc=None, root=None):
         """A class representing a Task
 
         :param connector: A Connector instance
@@ -73,10 +73,11 @@ class Task(base.ResourceBase):
         :param registries: Dict of Redfish Message Registry objects to be
             used in any resource that needs registries to parse messages
         :param field_data: the data to use populating the fields
+        :param root: Sushy root object. Empty for Sushy root itself.
         """
         super(Task, self).__init__(
             connector, identity, redfish_version, registries,
-            json_doc=json_doc)
+            json_doc=json_doc, root=root)
 
     @property
     def is_processing(self):
@@ -101,8 +102,8 @@ class TaskCollection(base.ResourceCollectionBase):
         """Summary of task ids and corresponding state
 
         :returns: dictionary in the format
-            {'jid_123456789': sushy.TASK_STATE_NEW,
-            'jid_123454321': sushy.TASK_STATE_RUNNING}
+            {'jid_123456789': sushy.TaskState.NEW,
+            'jid_123454321': sushy.TaskState.RUNNING}
         """
         task_dict = {}
         for task in self.get_members():
