@@ -15,6 +15,8 @@ from unittest import mock
 
 
 import sushy
+from sushy.resources import constants as res_cons
+from sushy.resources.system.storage import controller
 from sushy.resources.system.storage import drive
 from sushy.resources.system.storage import storage
 from sushy.resources.system.storage import volume
@@ -42,6 +44,12 @@ class StorageTestCase(base.TestCase):
         self.conn = mock.Mock()
         with open('sushy/tests/unit/json_samples/storage.json') as f:
             self.json_doc = json.load(f)
+        with open('sushy/tests/unit/json_samples/'
+                  'storage_controller_collection.json') as f:
+            self.json_doc_ctrl_col = json.load(f)
+        with open('sushy/tests/unit/json_samples/'
+                  'storage_controller.json') as f:
+            self.json_doc_ctrl = json.load(f)
 
         self.conn.get.return_value.json.return_value = self.json_doc
 
@@ -54,9 +62,9 @@ class StorageTestCase(base.TestCase):
         self.assertEqual('1.0.2', self.storage.redfish_version)
         self.assertEqual('1', self.storage.identity)
         self.assertEqual('Local Storage Controller', self.storage.name)
-        self.assertEqual('ok', self.storage.status.health)
-        self.assertEqual('ok', self.storage.status.health_rollup)
-        self.assertEqual('enabled', self.storage.status.state)
+        self.assertEqual(res_cons.Health.OK, self.storage.status.health)
+        self.assertEqual(res_cons.Health.OK, self.storage.status.health_rollup)
+        self.assertEqual(res_cons.State.ENABLED, self.storage.status.state)
         self.assertEqual(
             ('/redfish/v1/Systems/437XR1138R2/Storage/1/Drives/35D38F11ACEF7BD3',  # noqa
              '/redfish/v1/Systems/437XR1138R2/Storage/1/Drives/3F5A8C54207B7233',  # noqa
@@ -81,16 +89,20 @@ class StorageTestCase(base.TestCase):
         calls = [
             mock.call(self.storage._conn,
                       '/redfish/v1/Systems/437XR1138R2/Storage/1/Drives/35D38F11ACEF7BD3',  # noqa
-                      self.storage.redfish_version, None),
+                      redfish_version=self.storage.redfish_version,
+                      registries=None, root=None),
             mock.call(self.storage._conn,
                       '/redfish/v1/Systems/437XR1138R2/Storage/1/Drives/3F5A8C54207B7233',  # noqa
-                      self.storage.redfish_version, None),
+                      redfish_version=self.storage.redfish_version,
+                      registries=None, root=None),
             mock.call(self.storage._conn,
                       '/redfish/v1/Systems/437XR1138R2/Storage/1/Drives/32ADF365C6C1B7BD',  # noqa
-                      self.storage.redfish_version, None),
+                      redfish_version=self.storage.redfish_version,
+                      registries=None, root=None),
             mock.call(self.storage._conn,
                       '/redfish/v1/Systems/437XR1138R2/Storage/1/Drives/3D58ECBC375FD9F2',  # noqa
-                      self.storage.redfish_version, None)
+                      redfish_version=self.storage.redfish_version,
+                      registries=None, root=None)
         ]
         Drive_mock.assert_has_calls(calls)
         self.assertIsInstance(all_drives, list)
@@ -113,24 +125,33 @@ class StorageTestCase(base.TestCase):
         self.assertEqual(1, len(controllers))
         controller = controllers[0]
         self.assertEqual('0', controller.member_id)
-        self.assertEqual('Contoso Integrated RAID', controller.name)
-        self.assertEqual('ok', controller.status.health)
-        self.assertEqual('enabled', controller.status.state)
+        self.assertEqual(res_cons.Health.OK, controller.status.health)
+        self.assertEqual(res_cons.State.ENABLED, controller.status.state)
         identifiers = controller.identifiers
         self.assertIsInstance(identifiers, list)
         self.assertEqual(1, len(identifiers))
         identifier = identifiers[0]
-        self.assertEqual(sushy.DURABLE_NAME_FORMAT_NAA,
+        self.assertEqual(sushy.DurableNameFormat.NAA,
                          identifier.durable_name_format)
         self.assertEqual('345C59DBD970859C', identifier.durable_name)
         self.assertEqual(12, controller.speed_gbps)
-        self.assertEqual([sushy.PROTOCOL_TYPE_PCIe],
+        self.assertEqual([sushy.Protocol.PCIe],
                          controller.controller_protocols)
-        self.assertEqual([sushy.PROTOCOL_TYPE_SAS, sushy.PROTOCOL_TYPE_SATA],
+        self.assertEqual([sushy.Protocol.SAS, sushy.Protocol.SATA],
                          controller.device_protocols)
-        self.assertEqual([sushy.RAID_TYPE_RAID0, sushy.RAID_TYPE_RAID1],
+        self.assertEqual([sushy.RAIDType.RAID0, sushy.RAIDType.RAID1],
                          controller.raid_types)
 
+    def test_controllers(self):
+        self.conn.get.return_value.json.side_effect = [
+            self.json_doc_ctrl_col,
+            self.json_doc_ctrl]
+        controllers = self.storage.controllers
+        self.assertIsInstance(controllers, controller.ControllerCollection)
+        self.assertEqual(1, len(controllers.get_members()))
+        controller1 = controllers.get_members()[0]
+        self.assertEqual('1', controller1.identity)
+
     def test_drives_after_refresh(self):
         self.storage.refresh()
         self.conn.get.return_value.json.reset_mock()
@@ -258,7 +279,8 @@ class StorageCollectionTestCase(base.TestCase):
         Storage_mock.assert_called_once_with(
             self.stor_col._conn,
             '/redfish/v1/Systems/437XR1138R2/Storage/1',
-            self.stor_col.redfish_version, None)
+            redfish_version=self.stor_col.redfish_version, registries=None,
+            root=self.stor_col.root)
 
     @mock.patch.object(storage, 'Storage', autospec=True)
     def test_get_members(self, Storage_mock):
@@ -266,7 +288,8 @@ class StorageCollectionTestCase(base.TestCase):
         Storage_mock.assert_called_once_with(
             self.stor_col._conn,
             '/redfish/v1/Systems/437XR1138R2/Storage/1',
-            self.stor_col.redfish_version, None)
+            redfish_version=self.stor_col.redfish_version, registries=None,
+            root=self.stor_col.root)
         self.assertIsInstance(members, list)
         self.assertEqual(1, len(members))
 
