@@ -164,20 +164,28 @@ class ConnectorOpTestCase(base.TestCase):
         self.auth = mock_auth
         super(ConnectorOpTestCase, self).setUp()
         self.conn = connector.Connector(
-            'http://foo.bar:1234', verify=True)
+            'http://foo.bar:1234', verify=True,
+            server_side_retries=10, server_side_retries_delay=3)
         self.conn._auth = mock_auth
         self.data = {'fake': 'data'}
-        self.headers = {'X-Fake': 'header'}
+        self.headers = {'Accept-Encoding': 'identity', 'OData-Version': '4.0'}
         self.session = mock.Mock(spec=requests.Session)
         self.conn._session = self.session
         self.request = self.session.request
         self.request.return_value.status_code = http_client.OK
 
     def test_ok_get(self):
-        self.conn._op('GET', path='fake/path', headers=self.headers)
+        self.conn._op('GET', path='fake/path')
         self.request.assert_called_once_with(
             'GET', 'http://foo.bar:1234/fake/path',
-            headers=self.headers, json=None)
+            headers=self.headers, json=None, verify=True, timeout=60)
+
+    def test_ok_get_with_headers(self):
+        self.conn._op('GET', path='fake/path', headers={'answer': '42'})
+        self.request.assert_called_once_with(
+            'GET', 'http://foo.bar:1234/fake/path',
+            headers=dict(self.headers, answer='42'),
+            json=None, verify=True, timeout=60)
 
     def test_response_callback(self):
         mock_response_callback = mock.MagicMock()
@@ -187,33 +195,42 @@ class ConnectorOpTestCase(base.TestCase):
         self.assertEqual(1, mock_response_callback.call_count)
 
     def test_ok_get_url_redirect_false(self):
-        self.conn._op('GET', path='fake/path', headers=self.headers,
-                      allow_redirects=False)
+        self.conn._op('GET', path='fake/path', allow_redirects=False)
         self.request.assert_called_once_with(
             'GET', 'http://foo.bar:1234/fake/path',
-            headers=self.headers, json=None, allow_redirects=False)
+            headers=self.headers, json=None, allow_redirects=False,
+            verify=True, timeout=60)
 
     def test_ok_post(self):
-        self.conn._op('POST', path='fake/path', data=self.data.copy(),
-                      headers=self.headers)
+        self.conn._op('POST', path='fake/path', data=self.data.copy())
         self.request.assert_called_once_with(
             'POST', 'http://foo.bar:1234/fake/path',
-            json=self.data, headers=self.headers)
+            json=self.data,
+            headers=dict(self.headers, **{'Content-Type': 'application/json'}),
+            verify=True, timeout=60)
+
+    def test_ok_post_with_headers(self):
+        self.conn._op('POST', path='fake/path', data=self.data.copy(),
+                      headers={'answer': 42})
+        self.request.assert_called_once_with(
+            'POST', 'http://foo.bar:1234/fake/path',
+            json=self.data,
+            headers=dict(self.headers, **{'Content-Type': 'application/json',
+                                          'answer': 42}),
+            verify=True, timeout=60)
 
     def test_ok_put(self):
-        self.conn._op('PUT', path='fake/path', data=self.data.copy(),
-                      headers=self.headers)
+        self.conn._op('PUT', path='fake/path', data=self.data.copy())
         self.request.assert_called_once_with(
             'PUT', 'http://foo.bar:1234/fake/path',
-            json=self.data, headers=self.headers)
+            headers=dict(self.headers, **{'Content-Type': 'application/json'}),
+            json=self.data, verify=True, timeout=60)
 
     def test_ok_delete(self):
-        expected_headers = self.headers.copy()
-        expected_headers['OData-Version'] = '4.0'
-        self.conn._op('DELETE', path='fake/path', headers=self.headers.copy())
+        self.conn._op('DELETE', path='fake/path')
         self.request.assert_called_once_with(
             'DELETE', 'http://foo.bar:1234/fake/path',
-            headers=expected_headers, json=None)
+            headers=self.headers, json=None, verify=True, timeout=60)
 
     def test_ok_post_with_session(self):
         self.conn._session.headers = {}
@@ -225,7 +242,7 @@ class ConnectorOpTestCase(base.TestCase):
                       data=self.data)
         self.request.assert_called_once_with(
             'POST', 'http://foo.bar:1234/fake/path',
-            json=self.data, headers=expected_headers)
+            json=self.data, headers=expected_headers, verify=True, timeout=60)
         self.assertEqual(self.conn._session.headers,
                          {'X-Auth-Token': 'asdf1234'})
 
@@ -238,24 +255,25 @@ class ConnectorOpTestCase(base.TestCase):
         self.conn._op('GET', path=path, headers=headers)
         self.request.assert_called_once_with(
             'GET', 'http://foo.bar:1234' + path,
-            headers=expected_headers, json=None)
-
-    def test_odata_version_header_redfish_no_headers(self):
-        path = '/redfish/v1/bar'
-        expected_headers = {'OData-Version': '4.0'}
-        self.conn._op('GET', path=path)
-        self.request.assert_called_once_with(
-            'GET', 'http://foo.bar:1234' + path,
-            headers=expected_headers, json=None)
+            headers=expected_headers, json=None, verify=True, timeout=60)
 
     def test_odata_version_header_redfish_existing_header(self):
         path = '/redfish/v1/foo'
         headers = {'OData-Version': '3.0'}
-        expected_headers = dict(headers)
+        expected_headers = dict(self.headers, **headers)
         self.conn._op('GET', path=path, headers=headers)
         self.request.assert_called_once_with(
             'GET', 'http://foo.bar:1234' + path,
-            headers=expected_headers, json=None)
+            headers=expected_headers, json=None, verify=True, timeout=60)
+
+    def test_remove_header_accept_encoding(self):
+        path = '/redfish/v1/foo'
+        headers = {'Accept-Encoding': None}
+        self.headers.pop('Accept-Encoding')
+        self.conn._op('GET', path=path, headers=headers)
+        self.request.assert_called_once_with(
+            'GET', 'http://foo.bar:1234' + path,
+            headers=self.headers, json=None, verify=True, timeout=60)
 
     def test_timed_out_session_unable_to_create_session(self):
         self.conn._auth.can_refresh_session.return_value = False
@@ -333,6 +351,46 @@ class ConnectorOpTestCase(base.TestCase):
         self.auth.authenticate.assert_called_once()
         self.assertEqual(response.json, third_response.json)
 
+    def test_timed_out_session_fail_after_reestablish(self):
+        self.auth._session_key = 'asdf1234'
+        self.auth.get_session_key.return_value = 'asdf1234'
+        self.conn._auth = self.auth
+        self.session = mock.Mock(spec=requests.Session)
+        self.session.auth = None
+        self.conn._session = self.session
+        self.request = self.session.request
+        first_response = mock.MagicMock()
+        first_response.status_code = http_client.FORBIDDEN
+        second_response = mock.MagicMock()
+        second_response.status_code = http_client.BAD_REQUEST
+        self.request.side_effect = [first_response, second_response]
+        self.assertRaises(exceptions.BadRequestError,
+                          self.conn._op,
+                          'POST', path='fake/path', data=self.data,
+                          headers=self.headers)
+        self.auth.refresh_session.assert_called_with()
+        self.auth.can_refresh_session.assert_called_with()
+
+    def test_timed_out_session_fail_after_reestablish_no_recursion(self):
+        self.auth._session_key = 'asdf1234'
+        self.auth.get_session_key.return_value = 'asdf1234'
+        self.conn._auth = self.auth
+        self.session = mock.Mock(spec=requests.Session)
+        self.session.auth = None
+        self.conn._session = self.session
+        self.request = self.session.request
+        first_response = mock.MagicMock()
+        first_response.status_code = http_client.FORBIDDEN
+        second_response = mock.MagicMock()
+        second_response.status_code = http_client.FORBIDDEN
+        self.request.side_effect = [first_response, second_response]
+        self.assertRaises(exceptions.AccessError,
+                          self.conn._op,
+                          'POST', path='fake/path', data=self.data,
+                          headers=self.headers)
+        self.auth.refresh_session.assert_called_with()
+        self.auth.can_refresh_session.assert_called_with()
+
     def test_connection_error(self):
         self.request.side_effect = requests.exceptions.ConnectionError
         self.assertRaises(exceptions.ConnectionError, self.conn._op, 'GET')
@@ -361,6 +419,7 @@ class ConnectorOpTestCase(base.TestCase):
         self.assertEqual(http_client.BAD_REQUEST, exc.status_code)
         self.assertIsNotNone(exc.body)
         self.assertIn('body submitted was malformed JSON', exc.detail)
+        self.assertEqual(len(exc.extended_info), 3, exc.extended_info)
 
     def test_known_http_error_nonlist_ext_info(self):
         self.request.return_value.status_code =\
@@ -376,6 +435,8 @@ class ConnectorOpTestCase(base.TestCase):
         self.assertEqual(http_client.UNSUPPORTED_MEDIA_TYPE, exc.status_code)
         self.assertIsNotNone(exc.body)
         self.assertIn('See Resolution for information', exc.detail)
+        self.assertIn('unsupported media type',
+                      exc.extended_info['Resolution'])
 
     @mock.patch('time.sleep', autospec=True)
     def test_not_found_error(self, mock_sleep):
@@ -404,6 +465,102 @@ class ConnectorOpTestCase(base.TestCase):
         self.assertEqual(10, mock_sleep.call_count)
         self.assertEqual(11, self.request.call_count)
 
+    @mock.patch('time.sleep', autospec=True)
+    def test_op_retry_on_server_500_sys518(self, mock_sleep):
+        response_info = {"error": {"@Message.ExtendedInfo": [
+            {'MessageId': 'IDRAC.2.7.SYS518'}]}}
+        mock_error = mock.Mock()
+        mock_error.status_code = 500
+        mock_error.json.return_value = response_info
+        self.request.return_value.status_code = (
+            http_client.INTERNAL_SERVER_ERROR)
+        self.request.return_value.json.side_effect =\
+            exceptions.ServerSideError(
+                method='DELETE', url='http://foo.bar', response=mock_error)
+
+        self.assertRaises(exceptions.ServerSideError, self.conn._op, 'DELETE',
+                          'http://foo.bar')
+        self.assertEqual(10, mock_sleep.call_count)
+        self.assertEqual(11, self.request.call_count)
+
+    @mock.patch('time.sleep', autospec=True)
+    def test_op_retry_on_server_500_other_than_sys518(self, mock_sleep):
+        response_info = {"error": {"@Message.ExtendedInfo": [
+            {'MessageId': 'IDRAC.2.7.SYS999'}]}}
+        mock_error = mock.Mock()
+        mock_error.status_code = 500
+        mock_error.json.return_value = response_info
+        self.request.return_value.status_code = (
+            http_client.INTERNAL_SERVER_ERROR)
+        self.request.return_value.json.side_effect =\
+            exceptions.ServerSideError(
+                method='DELETE', url='http://foo.bar', response=mock_error)
+
+        self.assertRaises(exceptions.ServerSideError, self.conn._op, 'DELETE',
+                          'http://foo.bar')
+        self.assertEqual(0, mock_sleep.call_count)
+        self.assertEqual(1, self.request.call_count)
+
+    @mock.patch('time.sleep', autospec=True)
+    def test_op_retry_on_server_400_ilo_not_ready(self, mock_sleep):
+        response_info = {"error": {"@Message.ExtendedInfo": [
+            {'MessageId': 'iLO.2.15.InvalidOperationForSystemState'}]}}
+        mock_error = mock.Mock()
+        mock_error.status_code = 400
+        mock_error.json.return_value = response_info
+        self.request.return_value.status_code = (
+            http_client.INTERNAL_SERVER_ERROR)
+        self.request.return_value.json.side_effect =\
+            exceptions.ServerSideError(
+                method='DELETE', url='http://foo.bar', response=mock_error)
+
+        self.assertRaises(exceptions.ServerSideError, self.conn._op, 'DELETE',
+                          'http://foo.bar')
+        self.assertEqual(10, mock_sleep.call_count)
+        self.assertEqual(11, self.request.call_count)
+
+    @mock.patch('time.sleep', autospec=True)
+    def test_op_retry_on_server_400_ilo_not_ready_other_error(self,
+                                                              mock_sleep):
+        response_info = {"error": {"@Message.ExtendedInfo": [
+            {'MessageId': 'iLO.Invalid'}]}}
+        mock_error = mock.Mock()
+        mock_error.status_code = 400
+        mock_error.json.return_value = response_info
+        self.request.return_value.status_code = (
+            http_client.INTERNAL_SERVER_ERROR)
+        self.request.return_value.json.side_effect =\
+            exceptions.ServerSideError(
+                method='DELETE', url='http://foo.bar', response=mock_error)
+
+        self.assertRaises(exceptions.ServerSideError, self.conn._op, 'DELETE',
+                          'http://foo.bar')
+        self.assertEqual(0, mock_sleep.call_count)
+        self.assertEqual(1, self.request.call_count)
+
+    def test_op_retry_without_identity(self):
+        self.request.side_effect = [
+            mock.Mock(status_code=http_client.NOT_ACCEPTABLE),
+            mock.Mock(status_code=http_client.OK),
+        ]
+        self.conn._op('GET', 'http://foo.bar')
+
+        self.assertEqual(2, self.request.call_count)
+        headers_no_accept = self.headers.copy()
+        headers_no_accept.pop('Accept-Encoding')
+        self.request.assert_has_calls([
+            mock.call('GET', 'http://foo.bar', headers=self.headers,
+                      json=None, verify=True, timeout=60),
+            mock.call('GET', 'http://foo.bar', headers=headers_no_accept,
+                      json=None, verify=True, timeout=60),
+        ])
+
+    def test_op_retry_without_identity_fails(self):
+        self.request.return_value.status_code = http_client.NOT_ACCEPTABLE
+        self.assertRaises(exceptions.NotAcceptableError, self.conn._op,
+                          'GET', 'http://foo.bar')
+        self.assertEqual(2, self.request.call_count)
+
     def test_access_error(self):
         self.conn._auth = None
 
@@ -503,7 +660,7 @@ class ConnectorOpTestCase(base.TestCase):
             'this is expected prior to authentication', 'HTTP GET '
             'http://redfish/v1/SessionService returned code '
             '%s. unknown error Extended information: '
-            'none' % http_client.FORBIDDEN)
+            'None' % http_client.FORBIDDEN)
         self.assertEqual(http_client.FORBIDDEN, exc.status_code)
 
     def test_blocking_no_location_header(self):
@@ -539,10 +696,10 @@ class ConnectorOpTestCase(base.TestCase):
     def test_access_error_basic_auth(self, mock_log):
         self.conn._auth.can_refresh_session.return_value = False
         self.conn._session.auth = ('foo', 'bar')
-        self.request.return_value.status_code = http_client.FORBIDDEN
+        self.request.return_value.status_code = 403
         mock_response = mock.Mock()
         mock_response.json.side_effect = ValueError('no json')
-        mock_response.status_code = http_client.FORBIDDEN
+        mock_response.status_code = 403
         self.request.return_value.json.side_effect = ValueError('no json')
 
         with self.assertRaisesRegex(exceptions.AccessError,
@@ -551,12 +708,13 @@ class ConnectorOpTestCase(base.TestCase):
         exc = cm.exception
         self.assertEqual(http_client.FORBIDDEN, exc.status_code)
         self.conn._auth.authenticate.assert_not_called()
+        error_msg = (f'HTTP GET http://redfish/v1/SessionService '
+                     f'returned code {exc.status_code}. unknown error '
+                     f'Extended information: None')
         mock_log.assert_called_once_with(
             "We have encountered an AccessError when using 'basic' "
             "authentication. %(err)s",
-            {'err': 'HTTP GET http://redfish/v1/SessionService '
-                    'returned code HTTPStatus.FORBIDDEN. unknown error '
-                    'Extended information: none'}
+            {'err': error_msg}
         )
 
     def test__op_raises_connection_error(self):
@@ -596,8 +754,7 @@ class ConnectorOpTestCase(base.TestCase):
             '/redfish/v1/Systems/1',
             data={'Boot': {'BootSourceOverrideTarget': 'Cd',
                            'BootSourceOverrideEnabled': 'Once'}},
-            headers={'X-Fake': 'header',
-                     'If-Match': '"3d7b8a7360bf2941d"'},
+            headers=dict(self.headers, **{'If-Match': '"3d7b8a7360bf2941d"'}),
             blocking=False,
             timeout=60)
 
@@ -619,8 +776,8 @@ class ConnectorOpTestCase(base.TestCase):
             '/redfish/v1/Systems/1',
             data={'Boot': {'BootSourceOverrideTarget': 'Cd',
                            'BootSourceOverrideEnabled': 'Once'}},
-            headers={'X-Fake': 'header',
-                     'If-Match': 'W/"3d7b8a7360bf2941d"'},
+            headers=dict(self.headers,
+                         **{'If-Match': 'W/"3d7b8a7360bf2941d"'}),
             blocking=False,
             timeout=60)
 
@@ -640,7 +797,7 @@ class ConnectorOpTestCase(base.TestCase):
             '/redfish/v1/Systems/1',
             data={'Boot': {'BootSourceOverrideTarget': 'Cd',
                            'BootSourceOverrideEnabled': 'Once'}},
-            headers={'X-Fake': 'header'},
+            headers=self.headers,
             blocking=False,
             timeout=60)
 
@@ -668,8 +825,7 @@ class ConnectorOpTestCase(base.TestCase):
             '/redfish/v1/Systems/1',
             data={'Boot': {'BootSourceOverrideTarget': 'Cd',
                            'BootSourceOverrideEnabled': 'Once'}},
-            headers={'X-Fake': 'header',
-                     'If-Match': '"3d7b8a7360bf2941d"'},
+            headers=dict(self.headers, **{'If-Match': '"3d7b8a7360bf2941d"'}),
             blocking=False,
             timeout=60)
 
@@ -699,7 +855,7 @@ class ConnectorOpTestCase(base.TestCase):
             '/redfish/v1/Systems/1',
             data={'Boot': {'BootSourceOverrideTarget': 'Cd',
                            'BootSourceOverrideEnabled': 'Once'}},
-            headers={'X-Fake': 'header'},
+            headers=self.headers,
             blocking=False,
             timeout=60)
 
@@ -727,7 +883,7 @@ class ConnectorOpTestCase(base.TestCase):
             '/redfish/v1/Systems/1',
             data={'Boot': {'BootSourceOverrideTarget': 'Cd',
                            'BootSourceOverrideEnabled': 'Once'}},
-            headers={'X-Fake': 'header'},
+            headers=self.headers,
             blocking=False,
             timeout=60)
 
